import dune.perftool.blockstructured.quadrature
import dune.perftool.blockstructured.argument
import dune.perftool.blockstructured.geometry
import dune.perftool.blockstructured.spaces
import dune.perftool.blockstructured.basis
import dune.perftool.blockstructured.transformations
from dune.perftool.options import get_option
from dune.perftool.pdelab.quadrature import pymbolic_quadrature_position
from dune.perftool.blockstructured.spaces import lfs_inames
from dune.perftool.blockstructured.basis import (pymbolic_reference_gradient,
                                                 pymbolic_basis)
from dune.perftool.blockstructured.geometry import (pymbolic_jacobian_inverse_transposed,
                                                    pymbolic_jacobian_determinant,
                                                    pymbolic_facet_jacobian_determinant,
                                                    to_global)
from dune.perftool.blockstructured.tools import sub_element_inames
from dune.perftool.pdelab import PDELabInterface


class BlockStructuredInterface(PDELabInterface):
    def __init__(self):
        PDELabInterface.__init__(self)
        # register transformation
        from dune.perftool.generation import transform
        from dune.perftool. blockstructured.transformations import blockstructured_iname_duplication
        transform(blockstructured_iname_duplication)


    def generate_accumulation_instruction(self, expr, visitor):
        if get_option('vectorization_blockstructured'):
            from dune.perftool.blockstructured.accumulation import generate_accumulation_instruction
            return generate_accumulation_instruction(expr, visitor)
        else:
            return PDELabInterface.generate_accumulation_instruction(self, expr, visitor)
    #
    # Local function space related generator functions
    #
    # TODO current way to squeeze subelem iname in, not really ideal

    def lfs_inames(self, element, restriction, number=None, context=''):
        return sub_element_inames() + lfs_inames(element, restriction, number, context)

    #
    # Test and trial function related generator functions
    #

    def pymbolic_basis(self, element, restriction, number, context=''):
        return pymbolic_basis(element, restriction, number, context)

    def pymbolic_reference_gradient(self, element, restriction, number, context=''):
        return pymbolic_reference_gradient(element, restriction, number, context)

    #
    # Geometry related generator functions
    #

    def pymbolic_spatial_coordinate(self):
        return to_global(pymbolic_quadrature_position())

    def pymbolic_jacobian_determinant(self):
        return pymbolic_jacobian_determinant()

    def pymbolic_jacobian_inverse_transposed(self, i, j, restriction):
        return pymbolic_jacobian_inverse_transposed(i, j, restriction)

    def pymbolic_facet_jacobian_determinant(self):
        return pymbolic_facet_jacobian_determinant()
